package game.net ;


import java.io.* ;
import java.net.* ;
import java.util.logging.*;


/**
    This class is used by the client to get lines from the network.
*/
public class Client_network_layer
    implements Client_network_interface
{
// PUBLIC PART
    /** Construct a network layer object. */
    public Client_network_layer() { }
   
    /** Initialize the network layer.
        @param server_host_number_or_name This can be either of the form 134.153.12.70
                                   or of the form peta.engr.mun.ca.
        @param server_port this is the "well known port number" for the server.
        @exception java.net.UnknownHostException if the server host can not be found.
        @exception java.io.IOException if communication with the server can not be initialized.
    */
    public void init( String server_host_number_or_name, int server_port )
        throws java.net.UnknownHostException,
               java.io.IOException
    {
        if( initialized )
            throw new java.io.IOException("Layer already initialized" ) ;
        log.finer("Connecting to "+server_host_number_or_name) ;
        InetAddress addr = InetAddress.getByName(server_host_number_or_name) ;
        log.finer("Connecting to port "+server_port) ;
        client_socket = new Socket( addr, server_port ) ;
        log.finer("Connected") ;
        client_in_buffer = new Buffer( client_socket.getInputStream() ) ;
        client_out_stream = client_socket.getOutputStream() ;
        initialized = true ;
    }
     
    /** Send one line to the server.
        @see Client_network_interface 
    */
    public void send_line( String line )
        throws java.io.IOException
    {
        if( ! initialized )
            throw new java.io.IOException("Layer not initialized" ) ;
        client_out_stream.write( line.getBytes("ASCII") ) ;
        log.finest("Sending \"" + line + "\"") ;
    }
    
    /** Attempt to read one line from the server. 
        @see Client_network_interface */
    public boolean get_line( StringBuffer str_buf )
        throws java.io.IOException
    {
        if( ! initialized )
            throw new java.io.IOException("Layer not initialized" ) ;
            
        
        client_in_buffer.fill_with_available_characters() ;
        
        if( client_in_buffer.has_line() ) {
            client_in_buffer.deliver_line( str_buf ) ;
        	log.finest("Received \"" + str_buf + "\"");
            return true ; }
        else {
            return false ; }
    }
    
    /** Finish everything up
    @exception java.io.IOException if any trouble ;
    */
    public void shutdown()
        throws java.io.IOException 
    {
        if( ! initialized )
            throw new java.io.IOException("Layer not initialized" ) ;
        log.finer("Shutting down.");
        client_in_buffer.shutdown() ;
        client_out_stream.close() ;
        client_socket.close() ; }
        
// PRIVATE PART
    
    private Socket client_socket ;
    private Buffer client_in_buffer ;
    private OutputStream client_out_stream ;
    
    private boolean initialized = false ;
    
    private static Logger log = Logger.getLogger("Soccer.team");
    
}