/**
 * AccelerateRequest.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.*;

/**
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 */
public class AccelerateRequest extends ControlRequest
{
    public AccelerateRequest(String req) throws ParseException
    {
        String intRegex = "\\d{1,9}";
        String floatRegex = "-?" + intRegex + "\\." + intRegex;
        String regex = "accelerate! " + intRegex + " " + intRegex + " " + floatRegex + "\\n";
        if (!req.matches(regex))
        {
            throw new ParseException("Invalid request: " + req);
        }
        String[] comps = req.replaceAll("\\n", "").split(" ");
        mTime = Integer.parseInt(comps[1]);
        mPlayer = Integer.parseInt(comps[2]);
        mAccel = Double.parseDouble(comps[3]);


        mType = Request.RequestType.ACCELERATE;
    }

    public AccelerateRequest(int t, int p, double a) throws SCOREException
    {
    	if (t < 0 || p < 0 || p > 4 || a < -5 || a > 1) {
    		SCOREException se = new SCOREException("Invalid accelerate agument");
    		mClientLog.throwing("AccelerateRequest", "AccelerateRequest(int, int, double)", se);
    		throw se;
    	}
    	mTime   = t;
    	mPlayer = p;
    	mAccel  = a;
    	
    	mType = Request.RequestType.ACCELERATE;
    }

    public void dispatch(ISimulatorNet sim)
    {
        sim.accelerate(mSide, mPlayer, mAccel);
    }

    public String toString()
    {
        String result = "accelerate! ";
        result += String.format("%d %d %03.3f", mTime, mPlayer, mAccel);
        return result + "\n";
    }

    private double mAccel;
    private int    mPlayer;
}
