/**
 * ControlReply.java
 * @author dennis
 * Created: Feb 21, 2006
 */
package game.protocol;

/**
 * Normal response from a control request.
 * <p>Invariant: mType == CONTROL</p>
 */
public class ControlReply extends Reply {

	/**
	 * @param t time in milliseconds
	 */
	public ControlReply(int t) {
		msgTime = t;
		mType = ReplyType.CONTROL;
	}
	
	/**
	 * @param msg The message text
	 * @throws ParseException illformatted message.
	 */
	public ControlReply(String msg) throws ParseException {
		try {
			msgTime = new Integer(msg.substring(0, msg.length()-1));
		} catch (NumberFormatException e) {
			ParseException pe = new ParseException("Expected time", msg, 0, e);
			log.throwing("ControlReply", "ControlReply(String)", pe);
			throw pe;
		}
		mType = ReplyType.CONTROL;
	}
	/**
	 * @return time in milliseconds from message
	 */
	public int getTime() {
		return msgTime;
	}	
	
	public String toString() {
		String result = new Integer(msgTime).toString() + "\n";
		return result;
	}
	/** Timestamp from message. This represents the time that a request was received. */
	private int msgTime;
}
