/**
 * EndReply.java
 * @author dennis
 * Created: Feb 21, 2006
 */
package game.protocol;

import game.Side;


/**
 * Reply indicating that the game is over. Indicates the winner.
 * <p>Invariant: mType == ENDGAME</p>
 */
public class EndReply extends Reply {

	/**
	 * @param winner which side won. null indicates draw.
	 */
	public EndReply(Side winner) {
		mWinner = winner;
		mType = ReplyType.ENDGAME;
	}
	
	/**
	 * @param msg the message text
	 * @throws ParseException message format is incorrect.
	 */
	public EndReply(String msg) throws ParseException {
		if (msg.startsWith("end ")) {

			try {
				Integer win = new Integer(msg.substring(4, msg.length()-1));
				mWinner = (win == 2) ? null : Side.fromOrdinal(win);
				mType = ReplyType.ENDGAME;
			} catch (NumberFormatException e) {
				ParseException pe = new ParseException("Expected number", msg, 4, e);
				log.throwing("EndReply", "EndReply(String)", pe);
				throw pe;
			}
		} else {
			ParseException pe = new ParseException("Expected \"end\"", msg, 0);
			log.throwing("EndReply", "EndReply(String)", pe);
			throw pe;
		}
	}
	
	/**
	 * @return Winning side or null in the case of a draw.
	 */
	public Side getWinner() {
		return mWinner;
	}

	public String toString() {
		String result = "end " + 
					((mWinner !=null) ? new Integer(mWinner.ordinal()) : "2") + 
					"\n";
		return result;
	}
	
	/** Which side won the game. Null indicates a draw. */
	private Side mWinner;
	
}
