/**
 * GoalReply.java
 * @author dennis
 * Created: Feb 21, 2006
 */
package game.protocol;

import game.Side;

/** 
 * The first reply after a goal has been scored.  Gives the score for both sides.
 * <p>Invariant: mType == GOAL</p>
 */
public class GoalReply extends Reply {
	/**
	 * @param msg the message text
	 * @throws ParseException message format is incorrect.
	 */
	public GoalReply(String msg) throws ParseException {
		if (msg.startsWith("goal ")) {
			int tokenStart = 5;
			int tokenEnd = msg.indexOf(" ", tokenStart);
			try {
				westScore = new Integer(msg.substring(tokenStart, tokenEnd));
				tokenStart = tokenEnd+1;
				tokenEnd = msg.indexOf("\n", tokenStart);
				eastScore = new Integer(msg.substring(tokenStart,tokenEnd));
				mType = ReplyType.GOAL;
			} catch (NumberFormatException e) {
				ParseException pe = new ParseException("Invalid score", msg, tokenStart, e);
				log.throwing("GoalReply", "GoalReply(String)", pe);
				throw pe;
			}
		} else {
			ParseException pe = new ParseException("Expected \"goal \"", msg, 0);
			log.throwing("GoalReply", "GoalReply(String)", pe);
			throw pe;
		}
	}
	
	public GoalReply(int west, int east) {
		westScore = west;
		eastScore = east;
		mType = ReplyType.GOAL;
	}
	
	/** Get the new score for a team
	 * @param t the Team
	 * @return current score for t
	 */
	public int getScore(Side t) {
		switch (t) {
		case WEST:
			return westScore;
		case EAST:
			return eastScore;
		}
		return 0; // unreachable code, but Eclipse complains without it or default.
	}
	
	public String toString() {
		return "goal " + westScore + " " + eastScore + "\n";
	}
	
	private int westScore;
	
	private int eastScore;

}
