/**
 * KickRequest.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.*;

/**
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 *
 */
public class KickRequest extends ControlRequest
{
    public KickRequest(String req) throws ParseException
    {
        String intRegex = "\\d{1,9}";
        String floatRegex = "-?" + intRegex + "\\." + intRegex;
        String regex = "kick! " + intRegex + " " + intRegex + " " + floatRegex + " " + floatRegex + "\\n";
        if( !req.matches(regex) )
        {
            throw new ParseException("Invalid request: " + req);
        }
        String[] comps = req.replaceAll("\\n", "").split(" ");
        mTime   = Integer.parseInt(comps[1]);
        mPlayer = Integer.parseInt(comps[2]);
        mVel    = Double.parseDouble(comps[3]);
        mAlpha  = Double.parseDouble(comps[4]);

        mType = Request.RequestType.KICK;
    }

    public KickRequest(int t, int p, double v, double alpha) throws SCOREException
    {
    	if (t < 0 || p < 0 || p > 4 || v < 0 || v > 10 || alpha < -45 || alpha > 45) {
    		SCOREException se = new SCOREException("Invalid kick agument");
    		mClientLog.throwing("KickRequest", "KickRequest(int, int, double, double)", se);
    		throw se;
    	}
        mTime  = t;
        mPlayer = p;
        mVel    = v;
        mAlpha  = alpha;

        mType = Request.RequestType.KICK;
    }

    public void dispatch(ISimulatorNet sim)
    {
        sim.kick(mSide, mPlayer, mVel, mAlpha);
    }

    public double getAlpha()
    {
        return mAlpha;
    }

    public int getPlayer()
    {
        return mPlayer;
    }

    public double getVelocity()
    {
        return mVel;
    }

    public String toString()
    {
        String result = "kick! ";
        result += String.format("%d %d %03.3f %03.3f", mTime, mPlayer, mVel, mAlpha);
        return result + "\n";
    }

    private double mAlpha;
    private int    mPlayer;
    private double mVel;
}
