/**
 * OnSideReply.java
 * @author dennis
 * Created: Feb 21, 2006
 */
package game.protocol;

import game.Side;
/**
 * Informs team which side it is playing on.
 * <p>Invariant: mType == ONSIDE</p>
 */
public class OnSideReply extends Reply {

	/**
	 * @param msg text of the message
	 * @throws ParseException message format is incorrect.
	 */
	public OnSideReply(String msg) throws ParseException {
		if (msg.startsWith("on_side ")) {
			mType = ReplyType.ONSIDE;
			try {
				side = Side.fromOrdinal(new Integer(msg.substring(8, msg.length()-1)));
			} catch (NumberFormatException e) {
				ParseException pe = new ParseException("Invalid side.", msg, 8, e);
				log.throwing("OnSideReply", "OnSideReply(String)", pe);
				throw pe;
			}
		} else {
			ParseException pe = new ParseException("Expected \"on_side \"", msg, 0);
			log.throwing("OnSideReply", "OnSideReply(String)", pe);			
			throw pe;
		}
	}
	
	/**
	 * @param t side for this team
	 */
	public OnSideReply(Side t) {
		side = t;
		mType = ReplyType.ONSIDE;
	}
	
	public String toString() {
		return "on_side " + side.ordinal() + "\n";
	}
	
	/** Which side is assigned to this team
	 * @return EAST or WEST
	 */
	public Side getSide() {
		return side;
	}
	
	private Side side;
}
