/**
 * ParseException.java
 * @author dennis
 * Created: Mar 4, 2006
 */
package game.protocol;

/**
 * Error in parsing message.
 */
public class ParseException extends Exception {

	/**
	 * 
	 */
	public ParseException() {
		super();
		text = "<<unknown>>";
		offSet = 0;
	}

	/**
	 * @param msg
	 */
	public ParseException(String msg) {
		super(msg);
		text = "<<unknown>>";
		offSet = 0;
	}

	/**
	 * @param msg
	 * @param cause
	 */
	public ParseException(String msg, Throwable cause) {
		super(msg, cause);
		text = "<<unknown>>";
		offSet = 0;
	}

	/**
	 * @param cause
	 */
	public ParseException(Throwable cause) {
		super(cause);
		text = "<<unknown>>";
		offSet = 0;
	}
	
	/**
	 * @param txt text being parsed
	 * @param off offset, in text, of error
	 */
	public ParseException(String txt, int off) {
		super();
		text = txt;
		offSet = off;
	}
	
	/**
	 * @param msg detail description
	 * @param txt text being parsed
	 * @param off offset, in text, of error
	 */
	public ParseException(String msg, String txt, int off) {
		super(msg);
		text = txt;
		offSet = off;		
	}
	
	/**
	 * @param msg detail description
	 * @param txt text being parsed
	 * @param off offset, in text, of error
	 * @param cause another exception that caused this one.
	 */
	public ParseException(String msg, String txt, int off, Throwable cause) {
		super(msg, cause);
		text = txt;
		offSet = off;		
	}
	
	public String toString() {
		return super.toString() + " at [" + offSet + "] in \"" + text + "\"";
	}
	
	public String getText() {
		return text;
	}
	
	public int getOffset() {
		return offSet;
	}
	
	/** text being parsed */
	private String text;
	/** index of offending text */
	private int offSet;

}
