/**
 * PlaceBallRequest.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.*;

/**
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 */
public class PlaceBallRequest extends ControlRequest
{
    public PlaceBallRequest(String req) throws ParseException
    {
        String intRegex = "\\d{1,9}";
        String floatRegex = "-?" + intRegex + "\\." + intRegex;
        String regex = "place_ball! " + intRegex + " " + floatRegex + " " + floatRegex + "\\n";
        if( !req.matches(regex) )
        {
            throw new ParseException("Invalid request: " + req);
        }
        String[] comps = req.replaceAll("\\n", "").split(" ");
        mTime    = Integer.parseInt(comps[1]);
        double x = Double.parseDouble(comps[2]);
        double y = Double.parseDouble(comps[3]);
        mLoc = new CartesianCoord(x, y);

        mType = Request.RequestType.PLACEBALL;
    }

    public PlaceBallRequest(int t, CartesianVect l) throws SCOREException
    {
    	if (t < 0 || l.getX() < 0 || l.getX() > 50
    			|| l.getY() < 0 || l.getY() > 25) {
    		SCOREException se = new SCOREException("Invalid place_ball agument");
    		mClientLog.throwing("PlaceBallRequest", "PlaceBallRequest(int, CartesianVect)", se);
    		throw se;
    	}
        mTime = t;
        mLoc  = l;

        mType = Request.RequestType.PLACEBALL;
    }

    public void dispatch(ISimulatorNet sim)
    {
        sim.placeBall(mLoc);
    }

    public CartesianVect getLocation()
    {
        return mLoc;
    }

    public String toString()
    {
        String result = "place_ball! ";
        result += String.format("%d %03.3f %03.3f", mTime, mLoc.getX(), mLoc.getY());
        return result + "\n";
    }

    private CartesianVect mLoc;
}
