/**
 * PlacePlayerRequest.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.*;

/**
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 */
public class PlacePlayerRequest extends ControlRequest
{
    public PlacePlayerRequest(String req) throws ParseException
    {
        String intRegex = "\\d{1,9}";
        String floatRegex = "-?" + intRegex + "\\." + intRegex;
        String regex = "place_player! " + intRegex + " " + intRegex + " " + floatRegex + " " + floatRegex + "\\n";
        if( !req.matches(regex) )
        {
            throw new ParseException("Invalid request: " + req);
        }
        String[] comps = req.replaceAll("\\n", "").split(" ");
        mTime    = Integer.parseInt(comps[1]);
        mPlayer  = Integer.parseInt(comps[2]);
        double x = Double.parseDouble(comps[3]);
        double y = Double.parseDouble(comps[4]);
        mLoc = new CartesianCoord(x, y);

        mType = Request.RequestType.PLACEPLAYER;
    }

    public PlacePlayerRequest(int t, int p, CartesianVect l) throws SCOREException
    {
    	if (t < 0 || p < 0 || p > 4 || l.getX() < 0 || l.getX() > 50
    			|| l.getY() < 0 || l.getY() > 25) {
    		SCOREException se = new SCOREException("Invalid place_player agument");
    		mClientLog.throwing("PlacePlayerRequest", "PlacePlayerRequest(int, int, CartesianVect)", se);
    		throw se;
    	}
    	mTime   = t;
    	mPlayer = p;
    	mLoc    = l;
    	
    	mType = Request.RequestType.PLACEPLAYER;
    }

    public void dispatch(ISimulatorNet sim)
    {
        sim.placePlayer(mSide, mPlayer, mLoc);
    }

    public CartesianVect getLocation()
    {
        return mLoc;
    }

    public int getPlayer()
    {
        return mPlayer;
    }

    public String toString()
    {
        String result = "place_player! ";
        result += String.format("%d %d %03.3f %03.3f", mTime, mPlayer, mLoc.getX(), mLoc.getY());
        return result + "\n";
    }

    private CartesianVect mLoc;
    private int           mPlayer;
}
