/**
 * Reply.java
 * @author dennis
 * Created: Feb 21, 2006
 */
package game.protocol;

import java.util.logging.*;

/**
 * Abstraction of all replies that might come from the simulator. There is almost
 * nothing in common among the reply types, but any one of them might result from
 * a message to the simulator, so they are all extensions of this type.
 */
public abstract class Reply {

	public enum ReplyType { SOCCER2008, ONSIDE, QUIT, ALL, CONTROL, RESET, ENDGAME, GOAL };
	
	/**
	 * Factory for all replies.
	 * @param msg message text
	 * @throws ParseException message is incorrectly formatted.
	 */
	static public Reply factory(String msg) throws ParseException {
		Reply result = null;
		log.finest("Parsing \"" + msg + "\"");
		switch (msg.charAt(0)) {
		case 'S': // SOCCER2008
			result = new SOCCER2008Reply(msg);
			break;
		
		case 'o': // on_side
			result = new OnSideReply(msg);
			break;
		
		case 'q': // quit
			result = new QuitReply(msg);
			break;
			
		case 'r': // reset
			result = new ResetReply(msg);
			break;
			
		case 'e': // end
			result = new EndReply(msg);
			break;
			
		case 'g': // goal
			result = new GoalReply(msg);
			break;
			
		default: // control or all reply
			if (msg.length() < 8) { // control
				result = new ControlReply(msg);
			} else { // assume all
				result = new AllReply(msg);
			}
		}
		return result;
	}
	
	/** Get the type of this reply.
	 * @return the type of this reply
	 */
	public ReplyType getType() {
		return mType;
	}
	
	/** The reply type. */
	protected ReplyType mType;
	
	/** Most log messages go here */
	static protected Logger log = Logger.getLogger("Soccer.team");
	/** Log in mServerLog when the Reply is being constructed on the simulator side. */
	static protected Logger mServerLog = Logger.getLogger("Soccer.simulator");
}
