/**
 * Request.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.ISimulatorNet;
import java.util.logging.*;

/**
 * Abstraction of all requests that might come from a team. There is one type of
 * feedback request and several types of command requests all of which extend this type.
 */
public abstract class Request
{
    public enum RequestType { SOCCER2008, WANTSIDE, QUIT, ALL, SPIN, ACCELERATE, KICK, PLACEPLAYER, PLACEBALL, SETTIME, END }

    /**
     * Factory for all requests.
     * @param req request text
     */
    static public Request factory(String req) throws ParseException
    {
        Request result = null;
        log.finest("Parsing: \"" + req + "\"");
        switch (req.charAt(0)) {
        case 'S': // should be SOCCER2008
        	result = new SOCCER2008Request(req);
        	break;
        	
        case 'a': // accelerate or all
        	switch (req.charAt(1)) {
        	case 'l': // all
        		result = new AllRequest(req);
        		break;
        		
        	case 'c': // accelerate
        		result = new AccelerateRequest(req);
        		break;
        		
        	default:
            	ParseException e = new ParseException("Invalid request: " + req);
            	log.throwing("Request", "factory(String)", e);
            	throw e;
        	}
        	break;
        	
        case 'e': // end
        	result = new EndRequest(req);
        	break;

        case 'k': // kick
        	result = new KickRequest(req);
        	break;
        	
        case 'p': // place_player or place_ball
        	switch (req.charAt(6)) {
        	case 'p': // place_player
        		result = new PlacePlayerRequest(req);
        		break;
        		
        	case 'b': // place_ball
        		result = new PlaceBallRequest(req);
        		break;
        		
        	default:
            	ParseException e = new ParseException("Invalid request: " + req);
            	log.throwing("Request", "factory(String)", e);
            	throw e;
        	}
        	break;
        	
        case 'q': // quit
        	result = new QuitRequest(req);
        	break;
        	
        case 's': // spin or set_time
        	switch (req.charAt(1)) {
        	case 'p': // spin
        		result = new SpinRequest(req);
        		break;
        		
        	case 'e': // set_time
        		result = new SetTimeRequest(req);
        		break;
        		
        	default:
            	ParseException e = new ParseException("Invalid request: " + req);
            	log.throwing("Request", "factory(String)", e);
            	throw e;
        	}
        	break;
        	
        case 'w': // should be want_side
        	result = new WantSideRequest(req);
        	break;
        	
        default: 
        	ParseException e = new ParseException("Invalid request: " + req);
        	log.throwing("Request", "factory(String)", e);
        	throw e;
        	
        }
/*        
        if (req.startsWith("SOCCER2008"))
        {
            result = new SOCCER2008Request(req);
        }
        else if (req.startsWith("want_side"))
        {
            result = new WantSideRequest(req);
        }
        else if (req.startsWith("quit"))
        {
            result = new QuitRequest(req);
        }
        else if (req.startsWith("spin"))
        {
            result = new SpinRequest(req);
        }
        else if (req.startsWith("accelerate"))
        {
            result = new AccelerateRequest(req);
        }
        else if (req.startsWith("kick"))
        {
            result = new KickRequest(req);
        }
        else if (req.startsWith("place_player"))
        {
            result = new PlacePlayerRequest(req);
        }
        else if (req.startsWith("place_ball"))
        {
            result = new PlaceBallRequest(req);
        }
        else if (req.startsWith("set_time"))
        {
            result = new SetTimeRequest(req);
        }
        else if (req.startsWith("all"))
        {
            result = new AllRequest(req);
        }
        else if (req.startsWith("end"))
        {
            result = new EndRequest(req);
        }
        else
        {
        	ParseException e = new ParseException("Invalid request: " + req);
        	log.throwing("Request", "factory(String)", e);
            throw e;
        }*/
        return result;
    }

    /** Get the type of this request.
     * @return the type of this request
     */
    public RequestType getType()
    {
        return mType;
    }

    /** The request type. */
    protected RequestType mType;
    
    /** Most log messages are for simulator. */
    protected static Logger log = Logger.getLogger("Soccer.simulator");
    /** Log to mClientLog when request is being constructed on team */
    protected static Logger mClientLog = Logger.getLogger("Soccer.team");
}
