/**
 * SetTimeRequest.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.*;

/**
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 */
public class SetTimeRequest extends ControlRequest
{
    public SetTimeRequest(String req) throws ParseException
    {
        String intRegex = "\\d{1,9}";
        String regex = "set_time! " + intRegex + " " + intRegex + "\\n";
        if( !req.matches(regex) )
        {
            throw new ParseException("Invalid request: " + req);
        }
        String[] comps = req.replaceAll("\\n", "").split(" ");
        mTime    = Integer.parseInt(comps[1]);
        mSetTime = Integer.parseInt(comps[2]);

        mType = Request.RequestType.SETTIME;
    }

    public SetTimeRequest(int t, int setT) throws SCOREException
    {
    	if (t < 0 || setT < 0 || setT > 300000) {
    		SCOREException se = new SCOREException("Invalid set_time agument");
    		mClientLog.throwing("SetTimeRequest", "SetTimeRequest(int, int)", se);
    		throw se;
    	}
        mTime    = t;
        mSetTime = setT;

        mType = Request.RequestType.SETTIME;
    }

    public void dispatch(ISimulatorNet sim)
    {
        sim.setTime(mSetTime);
    }

    public int getSetTime()
    {
        return mSetTime;
    }

    public String toString()
    {
        String result = "set_time! ";
        result += String.format("%d %d", mTime, mSetTime);
        return result + "\n";
    }

    private int mSetTime;
}
