/**
 * Request.java
 * @author pouria
 * Created: March 4, 2006
 */
package game.protocol;

import simulator.model.*;

public class SpinRequest extends ControlRequest
{
    public SpinRequest(String req) throws ParseException
    {
        String intRegex = "\\d{1,9}";
        String floatRegex = "-?" + intRegex + "\\." + intRegex;
        String regex = "spin! " + intRegex + " " + intRegex + " " + floatRegex + "\\n";
        if( !req.matches(regex) )
        {
            throw new ParseException("Invalid request: " + req);
        }
        String[] comps = req.replaceAll("\\n", "").split(" ");
        mTime   = Integer.parseInt(comps[1]);
        mPlayer = Integer.parseInt(comps[2]);
        mAngVel = Double.parseDouble(comps[3]);

        mType = Request.RequestType.SPIN;
    }

    public SpinRequest(int t, int p, double av) throws SCOREException
    {
    	if (t < 0 || p < 0 || p > 4 || av < -90 || av > 90) {
    		SCOREException se = new SCOREException("Invalid spin agument");
    		mClientLog.throwing("SpinRequest", "SpinRequest(int, int, double)", se);
    		throw se;
    	}
        mTime   = t;
        mPlayer = p;
        mAngVel = av;

        mType = Request.RequestType.SPIN;
    }

    public void dispatch(ISimulatorNet sim)
    {
        sim.spin(mSide, mPlayer, mAngVel);
    }

    public double getAngularVelocity()
    {
        return mAngVel;
    }

    public int getPlayer()
    {
        return mPlayer;
    }

    public String toString()
    {
        String result = "spin! ";
        result += String.format("%d %d %03.3f", mTime, mPlayer, mAngVel);
        return result + "\n";
    }

    private double mAngVel;
    private int    mPlayer;
}
