/**
 * SMfacade.java
 * @author dennis
 * Created: Feb 15, 2006
 */
package simulator.model.debug;

import java.util.*;
import java.util.logging.*;

import game.*;
import game.net.INetSimulator;
import simulator.model.*;

/** A trivial implementation of the required simulator model interfaces for
 * debugging purposes. In most cases this simply logs the calls to a log file.
 * 
 * @author dpeters
 * @version $Revision: 384 $ $Date: 2008-03-11 09:55:27 -0230 (Tue, 11 Mar 2008) $
 *
 */
public class SMfacade implements ISimulatorNet {

	public SMfacade() {
		mTeams = new Student[2];
		mStatusMsg = "Debug model";
		mGameInProgress = false;
		
	}
	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#setNet(game.net.INetSimulator)
	 */
	public void setNet(INetSimulator n) {
		mNet = n;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#startGame()
	 */
	public void startGame() {
		log.info("Start Game.");
		mGameInProgress = true;
		mGameStartTime = new Date();
		setStatusMessage("Game started!");
	}

	
	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#endGame()
	 */
	@Override
	public void endGame() {
		log.finer("Game ended by network layer.");
		mGameInProgress = false;
		
	}
	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#accelerate(int, int, double)
	 */
	public void accelerate(Side side, int p, double a) {
		log.finer("accelerate(" + side + ", " + p + ", " + a + ")");
		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#kick(int, int, double, double)
	 */
	public void kick(Side side, int p, double v, double alpha) {
		log.finer("kick(" + side + ", " + p + ", " + v + ", " + alpha + ")");		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#placePlayer(int, int, simulator.model.CartesianVect)
	 */
	public void placePlayer(Side side, int p, CartesianVect l) {
		log.finer("placePlayer(" + side + ", " + p + ", (" + l.getX() + ", " + l.getY() + "))");		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#setTeams(game.Student, game.Student)
	 */
	public void setTeams(Student west, Student east) {
		log.finer("setTeams( " + west + ", " + east + ")");
		mTeams[Side.EAST.ordinal()] = east;
		mTeams[Side.WEST.ordinal()] = west;		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#spin(int, int, double)
	 */
	public void spin(Side side, int p, double av) {
		log.finer("spin(" + side + ", " + p + ", " + av + ")");		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#getISimulatorNet()
	 */
	public ISimulatorNet getISimulatorNet() {
		log.finest("getIsimulatorNet() returns this.");
		return this;
	}
	
	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#setTestMode(boolean)
	 */
	public void setTestMode(boolean m) {
		log.finer("setTestMode(" + m + ")");		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#field_name()
	 */
	public Student getFieldName() {
		log.finest("getFieldName() returns debug.");
		return Student.debug;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#status_message()
	 */
	public String getStatusMessage() {
		log.finest("getStatusMessage() returns \"" + mStatusMsg + "\"");
		return mStatusMsg;
	}

	public void setStatusMessage(String msg) {
		log.finer("setStatusMessage(\"" + msg + "\")");
		mStatusMsg = msg;
	}
	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#getGameState()
	 */
	public boolean isGameInProgress() {
		log.finest("isGameInProgress() returns " + mGameInProgress);
		return mGameInProgress;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#team_name(int)
	 */
	public Student getTeamName(Side t) {
		Student result = null;
		if (mTeams[t.ordinal()] != null) {
			result = mTeams[t.ordinal()];
		}
		log.finest("getTeamName(" + t + ") returns " + result);
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#team_score(int)
	 */
	public int getTeamScore(Side t) {
		int result = 0;
		log.finest("getTeamScore(" + t + ") returns " + result);
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#seconds_played()
	 */
	public int getTime() {
		int result = (mGameStartTime != null) ? (int)(new Date().getTime() - mGameStartTime.getTime()) : 0;
		log.finest("getTime() returns " + result);
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#player_position(int, int)
	 */
	public CartesianVect getPlayerPosition(Side t, int p) {
		int base = (t == Side.EAST) ? 0 : 1;
		SMVector result = SMVector.makeCartesian(base*25+10, (p+1)*4);
		log.finest("getPlayerPosition(" + t + ", " + p + 
				") returns (" + result.getX() + ", " + result.getY() + ")");
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#isPlayerInGame(int, int)
	 */
	public boolean isPlayerInGame(Side t, int p) {
		log.finest("isPlayerInGame(" + t + ", " + p +") returns true");
		return true;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#player_orientation(int, int)
	 */
	public PolarVect getPlayerVelocity(Side t, int p) {
		SMVector result = SMVector.makePolar(0, (t == Side.EAST) ? 0 : 180);
		log.finest("getPlayerVelocity(" + t + ", " + p + 
				") returns (" + result.getMagnitude() +
				" ang " + result.getAngle() + ")");
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#ball_position()
	 */
	public CartesianVect getBallPosition() {
		SMVector result = SMVector.makeCartesian(25, 13);
		log.finest("getBallPosition() returns (" + 
				result.getX() + ", " + result.getY() + ")");
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorView#getBallVelocity()
	 */
	public PolarVect getBallVelocity() {
		SMVector result = SMVector.makePolar(0, 0);
		log.finest("getBallVelocity() returns (" + 
				result.getMagnitude() +
				" ang " + result.getAngle() + ")");
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#placeBall(double, double)
	 */
	public void placeBall(CartesianVect l) {
		log.finest("placeBall((" + l.getX() + ", " + l.getY() + "))");		
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorNet#setTime(int)
	 */
	public void setTime(int t) {
		log.finer("setTime(" + t + ")");		
	}
	
	/** Representation of the teams playing the game. */
	private Student[] mTeams;

	/** Time of game start */
	private Date mGameStartTime;
	
	/** True iff game is running. */
	private boolean mGameInProgress;
	
	/** The status message. */
	private String mStatusMsg;
	
	/** Network layer. */
	private INetSimulator mNet;
		
	private static Logger log = Logger.getLogger("Soccer.simulator");

}
