/**
 * SVmain.java
 * @author dennis
 * Created: Feb 15, 2006
 */
package simulator.view.debug;

import java.awt.*;
import java.awt.event.*;
import java.util.logging.*;

import javax.swing.*;

import game.Side;
import simulator.model.*;
import simulator.view.*;

/** A simple text based view of the game that can be used to debug a model.
 * @author dennis
 * @version $Revision: 336 $ $Date: 2008-02-10 22:19:15 -0330 (Sun, 10 Feb 2008) $
 */
public class SVmain extends SVbase implements ActionListener {
	
	public SVmain() throws HeadlessException {
		super();
	}
	
	
	/**
	 * Construct the UI. This approach is used to avoid thread safety problems.
	 * @link http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html
	 */
	protected void createAndShowUI() {
	    setTitle("ENGI 9874 SOCCER Simulator Debug View");
	    setSize(WIDTH, HEIGHT);
	    Container pane = getContentPane();
	    JScrollPane fieldPane = new JScrollPane();
	    mField = new JTextArea(25, 40);
	    mField.setEditable(false);
	    mField.setLineWrap(true);
	    fieldPane.setViewportView(mField);
		pane.add(fieldPane, BorderLayout.CENTER);
		
	    mRefreshButton = new JButton("Refresh");
	    mRefreshButton.addActionListener(this);
	    pane.add(mRefreshButton, BorderLayout.SOUTH);
	    
	    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	    setVisible(true);
	    
		mUIReady = true; // must be last line of method 
	}
	

	public void setModel(simulator.model.ISimulatorView m) {
		log.finer("setModel(" + m.getFieldName() + ")");
		mModel = m;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == mRefreshButton) {
			log.finer("Refresh action.");
			mField.append("Refresh at " + mModel.getTime() + "ms\n");
			if (mModel.isGameInProgress()) {
				mField.append("Field: " + mModel.getFieldName() + "\n");
				mField.append("Status message: \"" + mModel.getStatusMessage() + "\"\n");
				CartesianVect pos = mModel.getBallPosition();
				PolarVect vel = mModel.getBallVelocity();
				mField.append(String.format("Ball pos: (%f, %f) Speed: %f Heading: %f\n", 
						pos.getX(), pos.getY(), vel.getMagnitude(), vel.getAngle()));
				for (Side t : Side.values()) {
					mField.append("Side: " + t + ", Team: " + mModel.getTeamName(t) +
							", Score: " + mModel.getTeamScore(t) + "\n");
					for (int p = 0; p < ISimulatorView.NUMBER_OF_PLAYERS; p++) {
						mField.append("   Player " + p + ": ");
						if (mModel.isPlayerInGame(t, p)) {
							pos = mModel.getPlayerPosition(t, p);
							vel = mModel.getPlayerVelocity(t, p);
							mField.append(String.format(" at (%f, %f) Speed: %f Heading: %f\n", 
									pos.getX(), pos.getY(), vel.getMagnitude(), vel.getAngle()));
						} else {
							mField.append("Not in game.\n");
						}
					}
				}
			} else {
				mField.append("Game not in progress.\n");
			}
		}		
	}
	
	private JTextArea mField;

	private JButton mRefreshButton;
	
	private simulator.model.ISimulatorView mModel;
	
	/** Initial width of frame. */
	private static final int WIDTH = 820;
	
	/** Initial height of frame. */
	private static final int HEIGHT = 750;

	private static Logger log = Logger.getLogger("Soccer.simulator");
	

}
