/**
 * ITeam.java
 * @author dennis
 * Created: Feb 22, 2006
 */
package team;

import simulator.model.*;
import game.Side;
import game.Student;

/**
 * Interface for team component. 
 * 
 * <p>A correct team implementation must implement this interface.</p>
 */
public interface ITeam {
	/** Team is always in one of these states.  */
	public enum TeamStatus { 
		/** No game running (Init in protocol). */
		INIT, 
		/** a game is underway. */
		RUNNING, 
		/** not connected to host. */
		DONE };
	
	/**
	 * Set an observer to be notified of changes in status.
	 * @param o the observer
	 */
	public void setStatusObserver(ITeamObserver o);
	
	/**
	 * Get the current Team status.
	 * @return status
	 */
	public TeamStatus getStatus();
	
	/**
	 * Identify the creator of this team.
	 * @return the creator's login name.
	 */
	public Student getTeam();
	
	/**
	 * Set the interface to the simulator.
	 * @param t the team network layer implementation.
	 */
	public void setSimulator(ISimulatorTeam t);
	
	/**
	 * Start a game.
	 * <p>Precondition: simulator is connected to host.</p>
	 * @param prefSide my preferred side 
	 */
	public void start(Side prefSide);
	
	/**
	 * Abandon the game.
	 */
	public void stop();
}
