/**
 * TeamControllerUI.java
 * @author dpeters
 * Created: Mar 8, 2006
 */
package team.debug;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.util.logging.*;

import javax.swing.*;

import simulator.model.ISimulatorTeam;
import team.*;
import game.Side;
import game.protocol.*;
 

/**
 * @author dpeters
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 */
class TeamControllerUI extends JFrame implements Runnable, ActionListener {
	
	public TeamControllerUI(TeamMain main) {
		mMain = main;
		mUIReady = false;
		mWaitingToDie = false;
	    javax.swing.SwingUtilities.invokeLater(this);
	}
	
	/**
	 * @return true if UI has been created.
	 */
	public boolean uiIsReady() {
		return mUIReady;
	}

	public void setSimuator(ISimulatorTeam sim) {
		mSim = sim;
		if (uiIsReady()) repaint();
	}
	
	public void setSide(Side side) {
		mMySide = side;
		if (uiIsReady()) repaint();
	}
	/**
	 * Construct the UI. This approach is used to avoid thread safety problems.
	 * @see http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html
	 */
	private void createAndShowUI() {
		updateTitle();
	    setSize(WIDTH, HEIGHT);
	    Container pane = getContentPane();
	    GridBagLayout gridBag = new GridBagLayout();
		pane.setLayout(gridBag);
	    GridBagConstraints gbConst = new GridBagConstraints();
		
	    // control request area
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.weightx = 0.0;
		gbConst.gridwidth = 1;
	    JLabel reqLabel = new JLabel("Request: ");
		gridBag.setConstraints(reqLabel, gbConst);
		pane.add(reqLabel);
		
	    mRequestChooser = new JComboBox(mPlayerReq);
		gridBag.setConstraints(mRequestChooser, gbConst);
		pane.add(mRequestChooser);
		
		JLabel timeLabel = new JLabel("Time: ");
		gridBag.setConstraints(timeLabel, gbConst);
		pane.add(timeLabel);
		
	    mTimeField = new JTextField(6);
		gbConst.weightx = 1.0;
		gridBag.setConstraints(mTimeField, gbConst);
		mTimeField.setText("0");
		pane.add(mTimeField);
		
	    JLabel playerLabel = new JLabel("Player: ");
		gbConst.weightx = 0.0;
		gridBag.setConstraints(playerLabel, gbConst);
		pane.add(playerLabel);
		
	    mPlayerChooser = new JComboBox(mPlayers);
		gridBag.setConstraints(mPlayerChooser, gbConst);
		pane.add(mPlayerChooser);
		
	    JLabel argLabel = new JLabel("Argument(s): ");
		gridBag.setConstraints(argLabel, gbConst);
		pane.add(argLabel);
		
	    mArgField = new JTextField(12);
		gbConst.weightx = 2.0;
	    gbConst.gridwidth = 2;
		gridBag.setConstraints(mArgField, gbConst);
		pane.add(mArgField);
		
		mReqSendButton = new JButton("Send request");
		mReqSendButton.addActionListener(this);
		gbConst.weightx = 0.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(mReqSendButton, gbConst);
		pane.add(mReqSendButton);
			    
		// End, All, Quit buttons
	    mEndButton = new JButton("End");
	    mEndButton.addActionListener(this);
		gbConst.gridwidth = 5;
		gridBag.setConstraints(mEndButton, gbConst);
	    pane.add(mEndButton);
	    
	    mAllButton = new JButton("All");
	    mAllButton.addActionListener(this);
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(mAllButton, gbConst);
	    pane.add(mAllButton);
	    
	    JScrollPane msgPane = new JScrollPane();
	    mMsgArea = new JTextArea(5, 40);
	    mMsgArea.setEditable(false);
	    mMsgArea.setLineWrap(true);
	    msgPane.setViewportView(mMsgArea);
		gbConst.weightx = 10.0;
		gbConst.weighty = 3.0;
		gridBag.setConstraints(msgPane, gbConst);
		pane.add(msgPane);
	    
	    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	    setVisible(true);
	    mUIReady = true;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		createAndShowUI();
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		Reply repl = null;
		try {
			if (src == mEndButton) {
				log.finer("End action.");
				mMsgArea.append("Sending end.\n");
				repl = mSim.end();
				if (repl.getType() == Reply.ReplyType.ENDGAME) {
					mMsgArea.append("Got end reply\n");
					mMain.setStatus(ITeam.TeamStatus.INIT);
				} else {
					mMsgArea.append("Got reply type: " + repl.getType() + "\n");
					updateStatus(repl);
				}
			} else if (src == mAllButton) {
				log.finer("All request.");
				mMsgArea.append("Sending all?\n");
				repl = mSim.getGameData();
				if (repl.getType() == Reply.ReplyType.ALL) {
					mMsgArea.append("Got all reply at " + ((AllReply)repl).getTime() + "\n");
				} else {
					mMsgArea.append("Got reply type: " + repl.getType() + "\n");
				}
				updateStatus(repl);
			} else if (src == mReqSendButton) {
				doControlRequest();
			}
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (ParseException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		// TODO Auto-generated method stub
		
	}
	
	private void updateStatus(Reply repl) {
		Reply.ReplyType type = repl.getType();
		if (type == Reply.ReplyType.ENDGAME) {
			mMain.setStatus(ITeam.TeamStatus.DONE);
		} else if (type == Reply.ReplyType.QUIT) {
			// Should actually never happen
			mMain.setStatus(ITeam.TeamStatus.INIT); 
		}
	}
	
	private Reply doControlRequest() throws IOException, ParseException {
		Reply repl = null;
		int player = mPlayerChooser.getSelectedIndex();
		String args = mArgField.getText().trim();
		double arg1;
		double arg2;
		String msg;
		int spcLoc;
		try {
			int tm = Integer.parseInt(mTimeField.getText());
			switch (mRequestChooser.getSelectedIndex()) {
			// Depends on order listed in mPlayerReq
			case 0: // Spin
				arg1 = Double.parseDouble(args);
				msg = String.format("Sending spin %d %d %f", tm, player, arg1);
				log.fine(msg);
				mMsgArea.append(msg + "\n");
				repl = mSim.spin(tm, player, arg1);
				break;
			
			case 1: // Accelerate
				arg1 = Double.parseDouble(args);
				msg = String.format("Sending accelerate %d %d %f", tm, player, arg1);
				log.fine(msg);
				mMsgArea.append(msg + "\n");
				repl = mSim.accelerate(tm, player, arg1);
				break;
				
			case 2: // Kick
				spcLoc = args.indexOf(" ");
				arg1 = Double.parseDouble(args.substring(0, spcLoc));
				arg2 = Double.parseDouble(args.substring(spcLoc+1));
				msg = String.format("Sending kick %d %d %f %f", tm, player, arg1, arg2);
				log.fine(msg);
				mMsgArea.append(msg + "\n");
				repl = mSim.kick(tm, player, arg1, arg2);
				break;
				
			case 3: // Place player
				spcLoc = args.indexOf(" ");
				arg1 = Double.parseDouble(args.substring(0, spcLoc));
				arg2 = Double.parseDouble(args.substring(spcLoc+1));
				msg = String.format("Sending place_player %d %d %f %f", tm, player, arg1, arg2);
				log.fine(msg);
				mMsgArea.append(msg + "\n");
				repl = mSim.place_player(tm, player, simulator.model.SMVector.makeCartesian(arg1, arg2));
				break;
			
			case 4: // Place ball
				spcLoc = args.indexOf(" ");
				arg1 = Double.parseDouble(args.substring(0, spcLoc));
				arg2 = Double.parseDouble(args.substring(spcLoc+1));
				msg = String.format("Sending place_ball %d %d %f %f", tm, player, arg1, arg2);
				log.fine(msg);
				mMsgArea.append(msg + "\n");
				repl = mSim.place_ball(tm, simulator.model.SMVector.makeCartesian(arg1, arg2));
				break;
				
			case 5: // Set time
				int newTm = Integer.parseInt(args);
				msg = String.format("Sending set_time %d %d", tm, newTm);
				log.fine(msg);
				mMsgArea.append(msg + "\n");
				repl = mSim.set_time(tm, newTm);
				break;
			}
			msg = "Got reply type: " + repl.getType();
			log.fine(msg);
			mMsgArea.append(msg + "\n");
		} catch (NumberFormatException e) {
			msg = "Invalid arguments: \"" + args;
			log.fine(msg);
			mMsgArea.append(msg + "\n");
		}
		updateStatus(repl);
		return repl;
	}
	
	private void updateTitle() {
		String title = "ENGI 9874 SOCCER Team UI (debug) -- ";
		if (mMySide != null) {
			title += mMySide;
		} else {
			title += "not playing.";
		}
		setTitle(title);		
	}
	
	private void setButtons() {
		boolean ok = (mSim != null && mMySide != null);
		mReqSendButton.setEnabled(ok);
		mEndButton.setEnabled(ok);
		mAllButton.setEnabled(ok);		
	}
	
	/**
	 * Ask this window to die when it's next painted. Window.dispose() doesn't
	 * work before the UI has been created. This is only needed when the decision
	 * to kill this window occurs before uiIsReady().
	 */
	public void pleaseDie() {
		log.finer("pleaseDie()");
		mWaitingToDie = true;
	}
	
	public void paint(Graphics g) {
		if (mWaitingToDie) { 
			log.finer("Closing UI.");
			this.dispose();
		} else {
			updateTitle();
			setButtons();
		}
		super.paint(g);
	}
	
	private boolean mWaitingToDie;
	
	/** Connection to simulator */
	private ISimulatorTeam mSim;
	/** The side that I'm playing on */
	private Side mMySide;
	
	private TeamMain mMain;
	
	/** True when the UI is constructed, false otherwise. */
	private boolean mUIReady;
	
	private JComboBox mRequestChooser;
	
	private JTextField mTimeField;
	
	private JComboBox mPlayerChooser;
	
	private JTextField mArgField;
	
	private JButton mReqSendButton;
	
	private JButton mEndButton;
	
	private JButton mAllButton;
	
//	private JButton mQuitButton;
	
	private JTextArea mMsgArea;
	
	private static String[] mPlayerReq = { "Spin", "Accelerate", "Kick", 
		                                     "Place player", "Place ball", "Set time" }; 
	
	private static String[] mPlayers = {"0", "1", "2", "3", "4"};
	
	/** Initial width of frame. */
	private static final int WIDTH = 610;
	
	/** Initial height of frame. */
	private static final int HEIGHT = 400;
	
	private static Logger log = Logger.getLogger("Soccer.team");

	
}
