/**
 * TeamMain.java
 * @author dpeters
 * Created: 2006.03.01
 */
package team.debug;

import java.io.IOException;
import java.util.logging.*;

import game.Side;
import game.Student;
import simulator.model.ISimulatorTeam;
import team.ITeam;
import team.ITeamObserver;
import game.protocol.*;

/**
 * @version $Revision: 316 $ $Date: 2008-01-22 23:16:24 -0330 (Tue, 22 Jan 2008) $
 */
public class TeamMain implements ITeam {

	public TeamMain() {
		mStatus = TeamStatus.INIT;
		mController = new TeamControllerUI(this);
	}
	
	/* (non-Javadoc)
	 * @see team.ITeam#setStatusObserver(team.ITeamObserver)
	 */
	public void setStatusObserver(ITeamObserver o) {
		mObs = o;
	}

	/* (non-Javadoc)
	 * @see team.ITeam#getStatus()
	 */
	public TeamStatus getStatus() {
		log.finer("getStatus() returns: " + mStatus);
		return mStatus;
	}

	/* (non-Javadoc)
	 * @see team.ITeam#getTeam()
	 */
	public Student getTeam() {
		log.finer("getTeam() returns Student.debug");
		return Student.debug;
	}

	/* (non-Javadoc)
	 * @see team.ITeam#setSimulator(simulator.model.ISimulatorTeam)
	 */
	public void setSimulator(ISimulatorTeam t) {
		log.finer("setSimulator()");
		mSim = t;
		mController.setSimuator(t);
	}


	/* (non-Javadoc)
	 * @see team.ITeam#start(game.Side)
	 */
	public void start(Side prefSide) {
		log.finer("start(" + prefSide + ")");
		if (getStatus() == TeamStatus.INIT) {
			try {
				Reply rep = mSim.initGame(prefSide);
				if (rep.getType() == Reply.ReplyType.ONSIDE) {
					mMySide = ((OnSideReply)rep).getSide();
					mController.setSide(mMySide);
					setStatus(TeamStatus.RUNNING);
				}
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}				
			
		}
		
	}

	/* (non-Javadoc)
	 * @see team.ITeam#stop()
	 */
	public void stop() {
		log.finer("Stop()");
		if (mSim != null) {
			try {
				if (mStatus == TeamStatus.RUNNING) {
					mSim.end();
				}
			} catch (Exception e) {
				log.fine("Caught exception in stop: " + e.toString());
			}
		}
		if (mController != null) {
			if (mController.uiIsReady()) {
				mController.dispose();
			} else {
				mController.pleaseDie();
			}
		}
		setStatus(TeamStatus.DONE);
	}
	

	void setStatus(TeamStatus s) {
		log.finer("setStatus(" + s + ")");
		mStatus = s;
		if (mObs != null) mObs.statusChange();
	}

	/** interface to simulator */
	private ISimulatorTeam mSim;
	/** object that registers to be interested in status changes */
	private ITeamObserver mObs;
	/** game status */
	private TeamStatus mStatus;
	
	private TeamControllerUI mController;
	
	/** What side am I on. */
	private Side mMySide;
	
	private static Logger log = Logger.getLogger("Soccer.team");
}
